package org.socialsketch.ui.imgpanel.example;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import javax.swing.JOptionPane;
import javax.swing.WindowConstants;
import org.socialsketch.ui.dialogcallbacks.AdvancedCallback;
import org.socialsketch.ui.dialogcallbacks.AdvancedCallback.FinishedSelectionEvent;
import org.socialsketch.ui.dialogcallbacks.SSDialogParams;
import org.socialsketch.ui.imgpanel.ImagePanel;
import org.socialsketch.ui.imgpanel.ImgRecord;

/**
 *
 * @author Dimitry Kireyenkov <dimitry@languagekings.com>
 */
public class SimpleGridView2 extends javax.swing.JFrame {
    private final AdvancedCallback mAdvancedCallback;

    /**
     * Creates new form SimpleGridView2
     * 
     * Needs advanced callback set
     * @param params evolvable with parameters.
     */
    public SimpleGridView2(SSDialogParams params) {
        
        initComponents();
        
        setDefaultCloseOperation(WindowConstants.HIDE_ON_CLOSE);
        
        addWindowListener(new SimpleWindowListener(params.getAdvancedCallback(), new FinishedSelectionEvent(null))); // when NULL means "cancelled" selection
        
        mAdvancedCallback =  params.getAdvancedCallback();
        
        
        
        // set up button listeners.
        btnShare.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                final AdvancedCallback.FinishedSelectionEvent finishedSelectionEvent = new AdvancedCallback.FinishedSelectionEvent(getSelectedFile());
                finishedSelectionEvent.setTweet(getTweet());
                SimpleGridView2.this.dispose();
                mAdvancedCallback.onFinishedSelection(finishedSelectionEvent);
            }

        });
        
        
        btnCancel.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
//                JOptionPane.showMessageDialog(SimpleGridView2.this, "Sharing of your creation cancelled");
                mAdvancedCallback.onFinishedSelection(new AdvancedCallback.FinishedSelectionEvent(null)); // initializing with null, means we aborted.
                SimpleGridView2.this.dispose();
            }
        });
        
        
        imagePanel1.loadImagesFromDir(params.getDirectoryWithImages());
        
        imagePanel1.addOnSelectionListener(new ImagePanel.IOnSelectionListener(){
            
            @Override
            public void onImageSelected(ImgRecord imgRec){
                lblPhotographLabel.setIcon(imgRec.getIcon());
            }
        });
    }
    
    /**
     * Returns selected image file file or NULL.
     * 
     * @return NULL if nothing is selected.
     */
    private File getSelectedFile() {
        ImgRecord rec = imagePanel1.getSelectedRecord();
        return rec.getOriginalImageFile();
    }
    
    
    
    /**
     * Returns typed in tweet.
     * 
     * @return "" empty string if there was nothing input.
     */
    private String getTweet(){
        return txtTweet.getText();
    }

    /**
     * This method is called from within the constructor to initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is always
     * regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jLabel1 = new javax.swing.JLabel();
        txtTweet = new javax.swing.JTextField();
        jLabel2 = new javax.swing.JLabel();
        btnShare = new javax.swing.JButton();
        btnCancel = new javax.swing.JButton();
        imagePanel1 = new org.socialsketch.ui.imgpanel.ImagePanel();
        lblPhotographLabel = new javax.swing.JLabel();

        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);

        jLabel1.setText("Choose your creation image:");

        txtTweet.setText("Your tweet #p5 void draw size setup");

        jLabel2.setText("Type your tweet");

        btnShare.setText("Share your creation!");

        btnCancel.setText("No, I don't want the world to see my genious");

        lblPhotographLabel.setText("This is photograph label");

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(txtTweet)
                    .addGroup(layout.createSequentialGroup()
                        .addComponent(btnShare, javax.swing.GroupLayout.PREFERRED_SIZE, 250, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addGap(18, 18, 18)
                        .addComponent(btnCancel, javax.swing.GroupLayout.PREFERRED_SIZE, 0, Short.MAX_VALUE)
                        .addGap(10, 10, 10))
                    .addGroup(layout.createSequentialGroup()
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(jLabel2)
                            .addComponent(jLabel1))
                        .addGap(0, 0, Short.MAX_VALUE))
                    .addComponent(imagePanel1, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addComponent(lblPhotographLabel, javax.swing.GroupLayout.PREFERRED_SIZE, 578, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(jLabel2)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(layout.createSequentialGroup()
                        .addComponent(txtTweet, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addGroup(layout.createSequentialGroup()
                                .addComponent(btnShare, javax.swing.GroupLayout.PREFERRED_SIZE, 54, javax.swing.GroupLayout.PREFERRED_SIZE)
                                .addGap(0, 0, Short.MAX_VALUE))
                            .addComponent(btnCancel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(jLabel1)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(imagePanel1, javax.swing.GroupLayout.PREFERRED_SIZE, 493, javax.swing.GroupLayout.PREFERRED_SIZE))
                    .addGroup(layout.createSequentialGroup()
                        .addGap(3, 3, 3)
                        .addComponent(lblPhotographLabel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)))
                .addContainerGap())
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents

    /**
     * @param args the command line arguments
     */
    public static void main(String args[]) {
        /* Set the Nimbus look and feel */
        //<editor-fold defaultstate="collapsed" desc=" Look and feel setting code (optional) ">
        /* If Nimbus (introduced in Java SE 6) is not available, stay with the default look and feel.
         * For details see http://download.oracle.com/javase/tutorial/uiswing/lookandfeel/plaf.html 
         */
        try {
            for (javax.swing.UIManager.LookAndFeelInfo info : javax.swing.UIManager.getInstalledLookAndFeels()) {
                if ("Nimbus".equals(info.getName())) {
                    javax.swing.UIManager.setLookAndFeel(info.getClassName());
                    break;
                }
            }
        } catch (ClassNotFoundException ex) {
            java.util.logging.Logger.getLogger(SimpleGridView2.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
        } catch (InstantiationException ex) {
            java.util.logging.Logger.getLogger(SimpleGridView2.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
        } catch (IllegalAccessException ex) {
            java.util.logging.Logger.getLogger(SimpleGridView2.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
        } catch (javax.swing.UnsupportedLookAndFeelException ex) {
            java.util.logging.Logger.getLogger(SimpleGridView2.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
        }
        //</editor-fold>

        /* Create and display the form */
        java.awt.EventQueue.invokeLater(new Runnable() {
            public void run() {
//                String C_TEST_DIR_WITH_IMAGES = "d:\\wamp\\www\\vocabuland\\img\\icons";
                String C_TEST_DIR_WITH_IMAGES = "c:\\Users\\Ernesto Guevara\\Pictures\\10 October";
                SSDialogParams params = new SSDialogParams();
                params.setDirectoryWithImages(new File(C_TEST_DIR_WITH_IMAGES));
                params.setAdvancedCallback(new AdvancedCallback() {

                    @Override
                    public void onFinishedSelection(AdvancedCallback.FinishedSelectionEvent evt) {
                        if ( evt.hasCancelled() ){
                            JOptionPane.showMessageDialog(null, "Your selection was cancelled");
                        }
                        else{
                            JOptionPane.showMessageDialog(null, "Your selection was selected: " + evt.getSelectedImagePath() + " Tweet: [" + evt.getTweet() + "]");
                            
                        }
                    }
                });
                
                new SimpleGridView2(params).setVisible(true);
            }
        });
    }

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton btnCancel;
    private javax.swing.JButton btnShare;
    private org.socialsketch.ui.imgpanel.ImagePanel imagePanel1;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel lblPhotographLabel;
    private javax.swing.JTextField txtTweet;
    // End of variables declaration//GEN-END:variables

    /**
     * This window listener instance simply catches "window close"
     * event and triggers the event to inform listener of canceled selection.
     */
    private class SimpleWindowListener extends WindowAdapter {
        private final AdvancedCallback mCallback;
        private final FinishedSelectionEvent mEvt;

        public SimpleWindowListener(AdvancedCallback callback, FinishedSelectionEvent evt) {
            mCallback = callback;
            mEvt = evt;
        }

        @Override
        public void windowClosing(WindowEvent e) {
            mCallback.onFinishedSelection(mEvt);
            SimpleGridView2.this.dispose();
//            super.windowClosed(e); //To change body of generated methods, choose Tools | Templates.
        }
        
        
    }
        
}
